@@ -3,13 +3,8 @@
  *
  * Copyright © 2015-2019 by Apple Inc.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * missing or damaged, see the license at "http://www.cups.org/".
- *
- * This file is subject to the Apple OS-Developed Software exception.
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  */
 
 /*
@@ -21,8 +16,9 @@
 #  include <CommonCrypto/CommonDigest.h>
 #elif defined(HAVE_GNUTLS)
 #  include <gnutls/crypto.h>
+#  include "md5-internal.h"
 #else
-#  include "md5-private.h"
+#  include "md5-internal.h"
 #endif /* __APPLE__ */
 
 
@@ -191,14 +187,23 @@ cupsHashData(const char    *algorithm,	/* I - Algorithm name */
   size_t	tempsize = 0;		/* Truncate to this size? */
 
 
-#  ifdef HAVE_GNUTLS_FIPS140_SET_MODE
-  unsigned oldmode = gnutls_fips140_mode_enabled();
-
-  gnutls_fips140_set_mode(GNUTLS_FIPS140_LAX, GNUTLS_FIPS140_SET_MODE_THREAD);
-#  endif /* HAVE_GNUTLS_FIPS140_SET_MODE */
-
   if (!strcmp(algorithm, "md5"))
-    alg = GNUTLS_DIG_MD5;
+  {
+   /*
+    * Some versions of GNU TLS disable MD5 without warning...
+    */
+
+    _cups_md5_state_t	state;		/* MD5 state info */
+
+    if (hashsize < 16)
+      goto too_small;
+
+    _cupsMD5Init(&state);
+    _cupsMD5Append(&state, data, datalen);
+    _cupsMD5Finish(&state, hash);
+
+    return (16);
+  }
   else if (!strcmp(algorithm, "sha"))
     alg = GNUTLS_DIG_SHA1;
   else if (!strcmp(algorithm, "sha2-224"))
@@ -234,10 +239,6 @@ cupsHashData(const char    *algorithm,	/* I - Algorithm name */
       gnutls_hash_fast(alg, data, datalen, temp);
       memcpy(hash, temp, tempsize);
 
-#  ifdef HAVE_GNUTLS_FIPS140_SET_MODE
-      gnutls_fips140_set_mode(oldmode, GNUTLS_FIPS140_SET_MODE_THREAD);
-#  endif /* HAVE_GNUTLS_FIPS140_SET_MODE */
-
       return ((ssize_t)tempsize);
     }
 
@@ -246,17 +247,9 @@ cupsHashData(const char    *algorithm,	/* I - Algorithm name */
 
     gnutls_hash_fast(alg, data, datalen, hash);
 
-#  ifdef HAVE_GNUTLS_FIPS140_SET_MODE
-    gnutls_fips140_set_mode(oldmode, GNUTLS_FIPS140_SET_MODE_THREAD);
-#  endif /* HAVE_GNUTLS_FIPS140_SET_MODE */
-
     return ((ssize_t)gnutls_hash_get_len(alg));
   }
 
-#  ifdef HAVE_GNUTLS_FIPS140_SET_MODE
-  gnutls_fips140_set_mode(oldmode, GNUTLS_FIPS140_SET_MODE_THREAD);
-#  endif /* HAVE_GNUTLS_FIPS140_SET_MODE */
-
 #else
  /*
   * No hash support beyond MD5 without CommonCrypto or GNU TLS...
@@ -266,6 +259,9 @@ cupsHashData(const char    *algorithm,	/* I - Algorithm name */
   {
     _cups_md5_state_t	state;		/* MD5 state info */
 
+    if (hashsize < 16)
+      goto too_small;
+
     _cupsMD5Init(&state);
     _cupsMD5Append(&state, data, datalen);
     _cupsMD5Finish(&state, hash);
@@ -290,10 +286,6 @@ cupsHashData(const char    *algorithm,	/* I - Algorithm name */
 
   too_small:
 
-#ifdef HAVE_GNUTLS_FIPS140_SET_MODE
-  gnutls_fips140_set_mode(oldmode, GNUTLS_FIPS140_SET_MODE_THREAD);
-#endif /* HAVE_GNUTLS_FIPS140_SET_MODE */
-
   _cupsSetError(IPP_STATUS_ERROR_INTERNAL, _("Hash buffer too small."), 1);
   return (-1);
 }
