@@ -1,16 +1,11 @@
 /*
  * File test program for CUPS.
  *
- * Copyright 2007-2018 by Apple Inc.
- * Copyright 1997-2007 by Easy Software Products.
+ * Copyright © 2007-2018 by Apple Inc.
+ * Copyright © 1997-2007 by Easy Software Products.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * missing or damaged, see the license at "http://www.cups.org/".
- *
- * This file is subject to the Apple OS-Developed Software exception.
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  */
 
 /*
@@ -118,12 +113,12 @@ main(int  argc,				/* I - Number of command-line arguments */
 #endif /* !_WIN32 */
 
    /*
-    * Count lines in psglyphs, rewind, then count again.
+    * Count lines in test file, rewind, then count again.
     */
 
-    fputs("\ncupsFileOpen(\"../data/media.defs\", \"r\"): ", stdout);
+    fputs("\ncupsFileOpen(\"testfile.txt\", \"r\"): ", stdout);
 
-    if ((fp = cupsFileOpen("../data/media.defs", "r")) == NULL)
+    if ((fp = cupsFileOpen("testfile.txt", "r")) == NULL)
     {
       puts("FAIL");
       status ++;
@@ -133,9 +128,9 @@ main(int  argc,				/* I - Number of command-line arguments */
       puts("PASS");
       fputs("cupsFileGets: ", stdout);
 
-      if ((count = count_lines(fp)) != 201)
+      if ((count = count_lines(fp)) != 477)
       {
-        printf("FAIL (got %d lines, expected 201)\n", count);
+        printf("FAIL (got %d lines, expected 477)\n", count);
 	status ++;
       }
       else
@@ -153,9 +148,9 @@ main(int  argc,				/* I - Number of command-line arguments */
 	  puts("PASS");
 	  fputs("cupsFileGets: ", stdout);
 
-	  if ((count = count_lines(fp)) != 201)
+	  if ((count = count_lines(fp)) != 477)
 	  {
-	    printf("FAIL (got %d lines, expected 201)\n", count);
+	    printf("FAIL (got %d lines, expected 477)\n", count);
 	    status ++;
 	  }
 	  else
