@@ -4,11 +4,7 @@
  * Copyright 2007-2016 by Apple Inc.
  * Copyright 1997-2006 by Easy Software Products.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * file is missing or damaged, see the license at "http://www.cups.org/".
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more information.
  */
 
 /*
@@ -248,9 +244,6 @@ cupsdAddCert(int        pid,		/* I - Process ID */
     fchown(fd, User, Group);
   }
 
-  DEBUG_printf(("ADD pid=%d, username=%s, cert=%s\n", pid, username,
-                cert->certificate));
-
   write(fd, cert->certificate, strlen(cert->certificate));
   close(fd);
 
@@ -284,9 +277,6 @@ cupsdDeleteCert(int pid)		/* I - Process ID */
 
       cupsdLogMessage(CUPSD_LOG_DEBUG2, "cupsdDeleteCert: Removing certificate for PID %d.", pid);
 
-      DEBUG_printf(("DELETE pid=%d, username=%s, cert=%s\n", cert->pid,
-                    cert->username, cert->certificate));
-
       if (prev == NULL)
         Certs = cert->next;
       else
